<?php

/**
 * Zur Konstruktion von Menues und Sitemaps
 */
class DLRGMenu {
	private $menuTable = null;
	private $currentId = "";
	
	/**
	 * Bei der Instanzierung der Klasse muss sirekt der Dateiname mitgegeben werden, weil zu diesem Zeitpunkt 
	 * die Datei direkt komplett eingelesen wird. Spaeter wird nur auf einem zweidimensionalen Array im Speicher gearbeitet.
	 * @param Dateiname
	 */
	public function __construct ($file, $currentId = "") {
		$this->menuTable = $this->loadMenu($file);
		$this->currentId = $currentId;
	}

	/** 
	 * Laed die komplette Menuedatei initial ein
	 */
	private function loadMenu($file){
		$fileContent = file($file);
		$menuTable=array();

		for($i=0; $i<count($fileContent); $i++) {
			if (substr($fileContent[$i],0,1) != '#') { 
				$row = explode("|",$fileContent[$i]);
				$menuTable[] = $row;
			}
		}
		return $menuTable;
	}
	
	public function renderCrumbs() {
		$content = '';
		$current = $this->currentId;
		$currLen = 0;
		foreach ($this->menuTable as $key => $menuItem) {
			$id = substr($menuItem[1], 1);
			if (substr($current,0,$currLen+2) == $id) {
				$content .= "&nbsp;&bull;&nbsp;";
				$content.= '<a href="'.$menuItem[3].'">'.$menuItem[4].'</a>';
				$currLen+=2;
			}
		}
		return $content;
	}
	
	public function renderMainMenu() {
		$content = '<ul>';
		foreach ($this->menuTable as $id => $menuItem) {
			if (trim($menuItem[0]) == "" && $menuItem[2] != "h") {
				$active = $this->isInSubMenu($menuItem[1], true);
				$content.= '<li>';
				$content.= ($active?'<strong>':'');
				$content.= '<a href="'.$menuItem[3].'">'.$menuItem[4].'</a>';
				$content.= ($active?'</strong>':'');
				$content.= '</li>';
			}
		}
		$content.= '</ul>';
		return $content;
	}
	
	public function renderSubmenu() {
		$content = '<ul intent in-mobile-after="#menu_main ul li strong" in-standard-prepend="#menu_sub">';
		$listen = array(); //Merken, welche Unterlisten auf welcher Ebene ge�ffnet wurden, um es wieder sauber zu schlie�en
		foreach ($this->menuTable as $id => $menuItem) {
			if (trim($menuItem[0]) != "" && $this->isInSubMenu($menuItem[1], true) && $this->isDirektSub($menuItem[1]) && $menuItem[2] != "h") {
				$lC = strlen($menuItem[1]);
				$i = $id+1;
				$nextMenuItem = $this->menuTable[$i];
				while($nextMenuItem && (trim($nextMenuItem[0]) == "" || !$this->isDirektSub($nextMenuItem[1]) || $nextMenuItem[2] == "h")) {
					$i++;
					$nextMenuItem = (isset($this->menuTable[$i])?$this->menuTable[$i]:null);
				}
				$lN = ($nextMenuItem!= null?strlen($nextMenuItem[1]):-1);
				$active = $this->isInSubMenu($menuItem[1]);
				$content.= '<li>';
				$content.= ($active?'<strong>':'');
				$content.= '<a href="'.$menuItem[3].'">'.$menuItem[4].'</a>';
				$content.= ($active?'</strong>':'');
				if ($lN > 0 && $lN > $lC) {
					$content.= '<ul>';
					$listen[$lN] = "1";
				} else {
					$content.= '</li>';
				} 
				if ($lN > 0 && $lN<$lC) {
					for ($i=$lC; $i > $lN; $i--) {
						if (isset($listen[$i])) {
							unset($listen[$i]);
							$content.= '</ul>';
						}
					}
					$content.= '</li>';
				} 
				$lV = $lC;
			}
		}
		$content.= '</ul>';
		return $content;
	}
	
	private function isDirektSub($id) {
		$id = substr($id, 1);
		$current = $this->currentId;

		// Spezial für Apps
		if (substr($current,0,2) == '21' && strlen ($current) > 2) {
			if (substr($current,0,4) !== substr($id,0,4) && substr($id,0,4) !== '2199') {
				return false;
			}
		}

		if (strlen($id) - strlen($current) > 2)  { // Ab 2. menüebene
			return false;
		} else if (strlen($id) - strlen($current) > 0) { // MEnü oberhalb vom Current aber zugehörig
			return strpos($id, $current) === 0;
		} else if (strlen($id) - strlen($current) == 0 && strlen($id) > 2) { // gleiche ebene
			$id = substr($id,0,strlen($id)-2);
			$current = substr($current,0,strlen($current)-2);
			return ($id==$current);
		}
		return true;
	}
	
	
	private function isInSubMenu($id, $main = false) {
		$id = substr($id, 1);
		$current = $this->currentId;
		if ($main) {
			$id = substr($id, 0,2);
			$current = substr($current, 0,2);
		} else {
			#echo $id . "," . $current . "#";
		}
		#echo (strpos($id, $current, 0) === 0?"current".$this->currentId . "#" .$id . "#" .  strpos($id, $this->currentId, 0)  ." ---- ":"");
		return strpos($current, $id, 0) === 0;
	}


	/**
	 * Methode zum Aufbau des Menues. Spezialbehandlung von Ebenenlink und Start-Eintrags.
	 * @param Parent Entry.
	 * @return HTML
	 */
	public function renderMenu($parentEntry=NULL) {
		$menu = "";
			
		if(isset($parentEntry) && $parentEntry != ""){
			$backlink = $this->getBackLink($parentEntry);
			$menu .= '<a class="menuitem top" href="'.$backlink[3].'">Eine Ebene h&ouml;her</a>';
			$menu .= '<a class="menuitem" id="'.$backlink[1].'" href="'.$backlink[3].'">'.$backlink[4].'</a>'."\n";
			$menu .= '<div id="s'.substr($backlink[1],1).'" class="menusub" >'."\n";
		}
		$menu .= $this->renderMenuHelper($parentEntry);
		if(isset($parentEntry) && $parentEntry != ""){
			$menu .= "</div>\n";
		}
		return $menu;
	}

	/**
	 * Rekursive Methode zum Aufbau des Menues mit Ausnahme des Ebenenlinks oder des Start-Eintrags.
	 * @param Parent Entry.
	 * @return HTML
	 */
	private function renderMenuHelper($parentEntry=NULL) {
		$menuTable = $this->getMenu($parentEntry);
		$menu = "";
		$i=0;

		while($i<count($menuTable)) {
			if(!isset($_GET["doc"]) || $_GET["doc"] ==""){$doc = "index";}else{$doc = $_GET["doc"];}
			$aktive = "index.php?doc=".$doc;
			if (count($menuTable[$i]) < 5 || !isset($menuTable[$i][5]) || strlen(trim($menuTable[$i][5])) == 0) {
				$checked = true;
				if (count ($menuTable[$i]) > 6 && trim($menuTable[$i][6]) != "") {
					eval("\$checked = " . $menuTable[$i][6]. ";");
				}
				if ($checked) {
					if($this->hasChildEntries(substr($menuTable[$i][1],1))) {
						$menu .= '<a class="menuitem" id="'.$menuTable[$i][1].'" href="'.$menuTable[$i][3].'">'.$menuTable[$i][4].'</a>'."\n";
						if ($this->currentId == "" || (strpos($this->currentId, substr($menuTable[$i][1],1)) === 0)) {
							$menu .= '<div id="s'.substr($menuTable[$i][1],1).'" class="menusub" >'."\n";
							$menu .= '   '.$this->renderMenuHelper(substr($menuTable[$i][1],1));
							$menu .= "</div>\n";
						}
					} else {
						if (strlen(trim ($menuTable[$i][3])) > 0) {
							$menu .= '<a class="menuitem" id="'.$menuTable[$i][1].'" href="'.$menuTable[$i][3].'">'.$menuTable[$i][4].'</a>'."\n";
						} else if ($menuTable[$i][4] == "---") {
							$menu .= '<div style="height:10px"></div>';							
						} else {
							$menu .= '<span class="menuitem" style="font-style:italic">'.$menuTable[$i][4].'</span>'."\n";
						}
					}
				}
			}
			$i++;
		}
		return $menu;
	}

	
	/**
	 * Ganze Menuedatei als Sitemap darstellen. Rekursive Methode!
	 * @param parentEntry fuer Rekursion)
	 * @return HTML.
	 */
	public function renderSitemap($parentEntry=NULL, $level = 0) {
		$menuTable = $this->getMenu($parentEntry);
		$menu = "";
		$i=0;
		
		$prefix = ($level == 0?'<h4 class="accordion_trigger">':"<li>");
		$posfix = ($level == 0?'</h4><div class="accordion_container">':"</li>");
		$po2fix = ($level == 0?'</div>':"");
		
		while($i<count($menuTable)) {
			if (count($menuTable[$i]) < 5 || !isset($menuTable[$i][5]) || strlen(trim($menuTable[$i][5])) == 0) {
				$checked = true;
				if (count ($menuTable[$i]) > 6 && trim($menuTable[$i][6]) != "") {
					eval("\$checked = " . $menuTable[$i][6]. ";");
				}
				if ($checked) {
					if($this->hasChildEntries(substr($menuTable[$i][1],1))) {
						$menu .= $prefix . '<a href="'.$menuTable[$i][3].'">'.$menuTable[$i][4].'</a>'.$posfix.'<ul>';
						$menu .= $this->renderSitemap(substr($menuTable[$i][1],1), $level +1);
						$menu .= "</ul>". $po2fix."\n";
					} else {
						if (strlen(trim ($menuTable[$i][3])) > 0) {
							$menu .= $prefix . '<a href="'.$menuTable[$i][3].'">'.$menuTable[$i][4].'</a>'. $posfix . $po2fix;
						} else {
							$menu .= '<span class="mapitem" style="font-style:italic">'.$menuTable[$i][4].'</span>'."\n";
						} 
					}
				}
			}
			$i++;
		}
		return $menu;
	}

	/**
	 * Submenue oder einfacher Eintrag?
	 */
	private function hasChildEntries($entryId) {
		$menuTable = $this->getMenu($entryId);
		if(count($menuTable)>0){
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Submenue separieren.
	 */
	private function getMenu($id=""){
		$menuTable = array();
		for($i=0; $i<count($this->menuTable); $i++) {
			if($this->menuTable[$i][0] == $id) {
				$menuTable[] = $this->menuTable[$i];
			}
		}

		return $menuTable;
	}

	/**
	 * Ebenenlink identifizieren.
	 */
	private function getBackLink($id){
		for($i=0; $i<count($this->menuTable); $i++) {
			if($this->menuTable[$i][1] == "i".$id) {
				return $this->menuTable[$i];
			}
		}

		return "";
	}
	
	public function hasAccess($docId = null) {
		if ($docId == null) return true;
		$i=0;
		$menuTable = $this->menuTable;
		while($i<count($menuTable)) {
			if ($docId == substr($menuTable[$i][1],1)) {
				return count($menuTable[$i]) < 5 || !isset($menuTable[$i][5]) || strlen(trim($menuTable[$i][5])) == 0;
			}
			$i++;
		}
		return false;
	}
}