<?php

// Einstellungen fuer PEAR::Cache_Lite
$options = array(
	'cacheDir' => '/tmp/',  // Verzeichnis zum Speichern der Cache-Dateien, muss vom Webserver beschreibbar sein
	'lifeTime' => 60*60*24*30, // Haltezeit in Sekunden (Sekunden*Minuten*Stunden*Tage)
	'controlType' => 'crc32'
);

/* Ab hier muss nichts mehr konfiguriert werden. */

if(!isset($_GET['path']) || !isset($_GET['layer']) || !preg_match("/^([a-z0-9]+\/)+[a-z0-9]+\.png$/i" , $_GET['path']))
{
	// Irgendwas stimmt mit der Anfrage nicht
	header("HTTP/1.0 400 Bad request");
	exit;
}

$path = $_GET['path'];
$etag = '"'.md5($path).'"';
	
require_once 'Cache/Lite.php'; 
$options['pearErrorMode'] = CACHE_LITE_ERROR_DIE;
$cache = new Cache_Lite($options);

// Cache-Schlssel ist der Etag-Wert
header('Cache-Control: ');
header('Pragma: ');
header("Content-type: image/png");
header('ETag: '.$etag);
header('Expires: '.date("D, d M Y H:i:s T", time()+$options['lifeTime']));
	
if($data = $cache->get($etag)) 
{
	// Kachel aus dem Cache holen
	header("X-OSMTILEPROXY-CACHE: HIT");

	// Wenn der Etag-Schlssel passt, hat der Browser die Kachel schon - Kachel nicht nochmal senden
	if(isset($_SERVER['HTTP_IF_NONE_MATCH']) && $_SERVER['HTTP_IF_NONE_MATCH'] === $etag) 
	{	
		header('HTTP/1.0 304 Not Modified');
		exit();
	}
	
	// Schlssel passt nicht, d.h. der Browser hat die Kachel nicht, also aus dem Proxy-Cache liefern
    echo $data;
}
else 
{ 	
	// Kachel ist nicht im Proxy-Cache, also von OSM holen
	$servers = array('OSM_OSMARENDER' => array( "http://a.tah.openstreetmap.org/",
												"http://b.tah.openstreetmap.org/",
												"http://c.tah.openstreetmap.org/"),
					 'OSM_CYCLEMAP' => array("http://a.andy.sandbox.cloudmade.com/",
											 "http://b.andy.sandbox.cloudmade.com/",
											 "http://c.andy.sandbox.cloudmade.com/"),
					 'OSM_MAPLINT' => array( "http://d.tah.openstreetmap.org/",
											 "http://e.tah.openstreetmap.org/",
											 "http://f.tah.openstreetmap.org/"),
					 'OSM_MAPNIK' => array(	 "http://a.tile.openstreetmap.org/",
											 "http://b.tile.openstreetmap.org/",
											 "http://c.tile.openstreetmap.org/") );

	if(!isset($servers[$_GET['layer']]))
	{
		header("HTTP/1.0 400 Bad request");
		exit;
	}

	// Server zufaellig waehlen
	$server = $servers[$_GET['layer']][rand(0,count($servers[$_GET['layer']])-1)];
	
	header("X-OSMTILEPROXY-PATH: ".$server.$path);
	
	if(false === ($data = file_get_contents($server.$path)))
	{
		// Irgendwas stimmt mit dem Ergebnis nicht; zum Debuggen den Inhalt von $data analysieren
		header("HTTP/1.0 502 Bad Gateway");
		exit;
	}

	header("X-OSMTILEPROXY-CACHE: MISS");
		
    echo $data;
	
	// Kachel fuer die Zukunft speichern.
    $cache->save($data);    
}